@@ -4,6 +4,7 @@ Async interface using asyncpg
 """
 
 import logging
+import os
 import re
 from collections import OrderedDict, defaultdict
 from random import randrange
@@ -13,8 +14,9 @@ import pandas as pd
 
 import ujson
 from firepit import get_storage, pgstorage
-from firepit.deref import auto_deref
-from firepit.exceptions import InvalidAttr, InvalidStixPath, UnknownViewname
+from firepit.deref import auto_deref_cached
+from firepit.exceptions import (InvalidAttr, InvalidStixPath, UnknownViewname,
+                                SessionExists, SessionNotFound)
 from firepit.pgstorage import (CHECK_FOR_COMMON_SCHEMA,
                                CHECK_FOR_QUERIES_TABLE, INTERNAL_TABLES,
                                LIKE_BIN, MATCH_BIN, MATCH_FUN, SUBNET_FUN,
@@ -23,8 +25,7 @@ from firepit.props import parse_prop, prop_metadata
 from firepit.query import Column, Limit, Offset, Order, Projection, Query
 from firepit.splitter import RecordList, shorten_extension_name
 from firepit.sqlstorage import (DB_VERSION, SqlStorage, _format_query,
-                                _get_col_dict, _make_aggs, _transform,
-                                get_path_joins)
+                                _make_aggs, _transform, get_path_joins)
 from firepit.validate import validate_name, validate_path
 
 logger = logging.getLogger(__name__)
@@ -68,6 +69,7 @@ class AsyncStorage:
                 await self.conn.execute(LIKE_BIN)
                 await self.conn.execute(SUBNET_FUN)
 
+        #TODO: fail if it already exists
         await self.conn.execute(f'CREATE SCHEMA IF NOT EXISTS "{self.session_id}"')
         await self.conn.execute(f'SET search_path TO "{self.session_id}", firepit_common')
 
@@ -86,6 +88,7 @@ class AsyncStorage:
         logger.debug('Attaching to storage for session %s', self.session_id)
         self.conn = await asyncpg.connect(self.connstring)
         await self.conn.execute(f'SET search_path TO "{self.session_id}", firepit_common')
+        #TODO: fail if it doesn't exist
 
     async def cache(self,
                     query_id: str,
@@ -165,8 +168,11 @@ class AsyncStorage:
 
     async def delete(self):
         """Delete ALL data in this store"""
-        stmt = (f'DROP SCHEMA "{self.session_id}" CASCADE')
-        await self.conn.execute(stmt)
+        try:
+            stmt = (f'DROP SCHEMA "{self.session_id}" CASCADE')
+            await self.conn.execute(stmt)
+        except asyncpg.exceptions.InvalidSchemaNameError as e:
+            raise SessionNotFound(self.session_id) from e
 
     async def set_appdata(self, viewname, data):
         """Attach app-specific data to a viewname"""
@@ -288,8 +294,9 @@ class AsyncStorage:
             qry.append(Projection(proj))
         else:
             if not col_dict:
-                col_dict = _get_col_dict(self)
-            joins, proj = auto_deref(col_dict, viewname, self.types())
+                col_dict = await self._get_col_dict()
+            dbcols = await self.columns(viewname)
+            joins, proj = auto_deref_cached(viewname, dbcols, col_dict)
             if joins:
                 qry.extend(joins)
             if proj:
@@ -359,12 +366,22 @@ class AsyncStorage:
                 await self._new_name(viewname, sco_type)
 
     async def _is_sql_view(self, name):
-        viewdef = self.conn.fetchone("SELECT definition"
-                                     " FROM pg_views"
-                                     " WHERE schemaname = %s"
-                                     " AND viewname = %s", self.session_id, name)
+        viewdef = await self.conn.fetchrow(
+            "SELECT definition"
+            " FROM pg_views"
+            " WHERE schemaname = $1"
+            " AND viewname = $2",
+            self.session_id, name)
         return viewdef is not None
 
+    async def _get_col_dict(self):
+        q = Query('__columns')
+        col_dict = defaultdict(list)
+        results = await self.run_query(q)
+        for result in results:
+            col_dict[result['otype']].append(result['path'])
+        return col_dict
+
 
 class AsyncSplitWriter:
     """
@@ -466,8 +483,7 @@ class AsyncSplitWriter:
                 logger.debug('Added new type %s', obj_type)
             except (asyncpg.exceptions.DuplicateTableError,
                     asyncpg.exceptions.UniqueViolationError,
-                    asyncpg.exceptions.DuplicateObjectError) as e:
-                logger.error('TEMP: %s', e)
+                    asyncpg.exceptions.DuplicateObjectError):
                 logger.debug('Failed to add %s; refreshing schemas', obj_type)
                 # Refresh schemas
                 loaded_schema = await self._load_schema(obj_type)
@@ -598,12 +614,14 @@ class AsyncSqlWriter:
         logger.debug('Writing %d %s objects (%d props)', len(records), obj_type, len(schema))
         tablename = f'{self.prefix}{obj_type}'
 
+        # Load records into dataframe and do type conversions as required
+        df = pd.DataFrame(records)
+        await self.write_df(tablename, df, query_id, schema)
+
+    async def write_df(self, tablename, df, query_id, schema):
         # Generate random tmp table name
         r = randrange(0, 1000000)
         tmp = f'tmp{r}_{tablename}'
-
-        # Load records into dataframe and do type conversions as required
-        df = pd.DataFrame(records)
         columns = df.columns
         for col in columns:
             if col not in schema:
@@ -672,7 +690,7 @@ class AsyncSqlWriter:
             try:
                 await self.conn.execute(stmt)
             except asyncpg.exceptions.CardinalityViolationError as e:
-                logger.error('CardinalityViolationError for %s', obj_type)
+                logger.error('CardinalityViolationError for %s', tablename)
                 logger.critical('%s', e, exc_info=e)
                 raise e
 
@@ -880,6 +898,7 @@ class SyncWrapper(AsyncStorage):
                  session_id: str = None,
                  store: SqlStorage = None):
         if store:
+            logger.debug('Wrapping storage object %s', store)
             self.store = store
             self.dialect = self.store.dialect
         else:
@@ -891,9 +910,12 @@ class SyncWrapper(AsyncStorage):
         """
         Create a new "session" (SQLite3 file).  Fail if it already exists.
         """
-        #TODO: Fail if it already exists
+        # Fail if it already exists
+        if os.path.exists(self.connstring):
+            raise SessionExists(self.connstring)
         logger.debug('Creating storage for session %s', self.session_id)
         self.store = get_storage(self.connstring, self.session_id)
+        self.conn = self.store.connection
         self.placeholder = self.store.placeholder
         self.dialect = self.store.dialect
 
@@ -978,7 +1000,10 @@ class SyncWrapper(AsyncStorage):
         """Get the value of `viewname`"""
         return self.store.lookup(viewname, cols, limit, offset, col_dict)
 
+    async def _is_sql_view(self, name):
+        return self.store._is_sql_view(name)
 
-async def get_async_storage(store: SqlStorage) -> SyncWrapper:
+
+async def get_async_storage(store: SqlStorage) -> SyncWrapper:  #FIXME: why is this async?
     """Wrap a sync SqlStorage object with an async interface"""
     return SyncWrapper(store=store)
