@@ -89,7 +89,7 @@ def _make_aggs(group_cols, sco_type, schema):
 def infer_type(key, value):
     if key == 'id':
         rtype = 'TEXT UNIQUE'
-    elif key in ['src_port', 'dst_port', 'x_firepit_rank']:
+    elif key in ['number_observed', 'src_port', 'dst_port', 'x_firepit_rank']:
         rtype = 'INTEGER'
     elif key == 'ipfix.flowId':
         rtype = 'TEXT'  # Should be uint64, but that's not supported anywhere!
